@@ -3,6 +3,8 @@ package com.daniel.mobilepauker2.activities;
 import android.Manifest;
 import android.app.Activity;
 import android.app.AlertDialog;
+import android.app.NotificationChannel;
+import android.app.NotificationManager;
 import android.app.SearchManager;
 import android.content.Context;
 import android.content.DialogInterface;
@@ -15,6 +17,7 @@ import android.preference.PreferenceManager;
 import android.provider.Settings;
 import android.support.annotation.NonNull;
 import android.support.annotation.Nullable;
+import android.support.v4.app.NotificationManagerCompat;
 import android.support.v7.app.AppCompatActivity;
 import android.support.v7.widget.LinearLayoutManager;
 import android.support.v7.widget.RecyclerView;
@@ -30,6 +33,7 @@ import com.daniel.mobilepauker2.PaukerManager;
 import com.daniel.mobilepauker2.R;
 import com.daniel.mobilepauker2.model.ModelManager;
 import com.daniel.mobilepauker2.model.SettingsManager;
+import com.daniel.mobilepauker2.model.notification.NotificationService;
 import com.daniel.mobilepauker2.statistics.ChartAdapter;
 import com.daniel.mobilepauker2.statistics.ChartAdapter.ChartAdapterCallback;
 import com.daniel.mobilepauker2.utils.Constants;
@@ -42,6 +46,9 @@ import static com.daniel.mobilepauker2.PaukerManager.showToast;
 import static com.daniel.mobilepauker2.model.ModelManager.LearningPhase.FILLING_USTM;
 import static com.daniel.mobilepauker2.model.ModelManager.LearningPhase.SIMPLE_LEARNING;
 import static com.daniel.mobilepauker2.model.SettingsManager.Keys.HIDE_TIMES;
+import static com.daniel.mobilepauker2.utils.Constants.NOTIFICATION_CHANNEL_ID;
+import static com.daniel.mobilepauker2.utils.Constants.REQUEST_CODE_SAVE_DIALOG_NORMAL;
+import static com.daniel.mobilepauker2.utils.Constants.TIMER_BAR_CHANNEL_ID;
 
 /**
  * Created by Daniel on 24.02.2018.
@@ -51,7 +58,9 @@ import static com.daniel.mobilepauker2.model.SettingsManager.Keys.HIDE_TIMES;
  */
 
 public class MainMenu extends AppCompatActivity {
-    private static final int RQ_WRITE_EXT = 99;
+    private static final int RQ_WRITE_EXT_SAVE = 98;
+    private static final int RQ_WRITE_EXT_OPEN = 99;
+    public static boolean isPaukerActive = false;
     private final ModelManager modelManager = ModelManager.instance();
     private final PaukerManager paukerManager = PaukerManager.instance();
     private final SettingsManager settingsManager = SettingsManager.instance();
@@ -63,6 +72,16 @@ public class MainMenu extends AppCompatActivity {
     @Override
     protected void onCreate(@Nullable Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
+
+        // Channel erstellen, falls noch nicht vorhanden
+        Log.d("AlamNotificationReceiver::onReceive", "Create Channels");
+        createNotificationChannels();
+
+        String action = getIntent().getAction();
+        if (action != null && action.equals("Open Lesson")) {
+            startActivity(new Intent(context, LessonImportActivity.class));
+        }
+
         ErrorReporter.instance().init(context);
         PreferenceManager.setDefaultValues(this, R.xml.preferences, false);
 
@@ -87,23 +106,22 @@ public class MainMenu extends AppCompatActivity {
         if (errorReporter.isThereAnyErrorsToReport()) {
             AlertDialog.Builder alt_bld = new AlertDialog.Builder(this);
 
-            alt_bld.setMessage(getString(R.string.crash_report_message));
-            alt_bld.setCancelable(false);
-            alt_bld.setPositiveButton(getString(R.string.ok), new DialogInterface.OnClickListener() {
-                public void onClick(DialogInterface dialog, int id) {
-                    errorReporter.CheckErrorAndSendMail();
-                }
-            });
-
-            alt_bld.setNegativeButton(getString(R.string.cancel), new DialogInterface.OnClickListener() {
-                public void onClick(DialogInterface dialog, int id) {
-                    errorReporter.deleteErrorFiles();
-                    dialog.cancel();
-                }
-            });
+            alt_bld.setTitle(getString(R.string.crash_report_title))
+                    .setMessage(getString(R.string.crash_report_message))
+                    .setCancelable(false)
+                    .setPositiveButton(getString(R.string.ok), new DialogInterface.OnClickListener() {
+                        public void onClick(DialogInterface dialog, int id) {
+                            errorReporter.CheckErrorAndSendMail();
+                        }
+                    })
+                    .setNeutralButton(getString(R.string.cancel), new DialogInterface.OnClickListener() {
+                        public void onClick(DialogInterface dialog, int id) {
+                            errorReporter.deleteErrorFiles();
+                            dialog.cancel();
+                        }
+                    });
 
             AlertDialog alert = alt_bld.create();
-            alert.setTitle(getString(R.string.crash_report_title));
             alert.setIcon(R.mipmap.ic_launcher);
             alert.show();
         }
@@ -205,14 +223,6 @@ public class MainMenu extends AppCompatActivity {
         startActivity(browseIntent);
     }
 
-    /**
-     * Startet die Permissionanfrage
-     */
-    private void requestPermission() {
-        requestPermissions(new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE},
-                RQ_WRITE_EXT);
-    }
-
     @Override
     public boolean onCreateOptionsMenu(Menu menu) {
         getMenuInflater().inflate(R.menu.main, menu);
@@ -289,37 +299,62 @@ public class MainMenu extends AppCompatActivity {
             invalidateOptionsMenu();
         }
         firstStart = false;
+
+        isPaukerActive = true;
+
+        NotificationManagerCompat notificationManager = NotificationManagerCompat.from(context);
+        notificationManager.cancelAll();
     }
 
     @Override
-    public void onRequestPermissionsResult(int requestCode, @NonNull String permissions[],
+    public void onBackPressed() {
+        if (paukerManager.isSaveRequired()) {
+            AlertDialog.Builder builder = new AlertDialog.Builder(context);
+            builder.setMessage(R.string.close_without_saving_dialog_msg)
+                    .setPositiveButton(R.string.cancel, null)
+                    .setNeutralButton(R.string.close, new DialogInterface.OnClickListener() {
+                        @Override
+                        public void onClick(DialogInterface dialog, int which) {
+                            finish();
+                        }
+                    });
+            AlertDialog dialog = builder.create();
+            dialog.show();
+            dialog.getButton(AlertDialog.BUTTON_NEUTRAL).setTextColor(getColor(R.color.unlearned));
+            dialog.getButton(AlertDialog.BUTTON_POSITIVE).setTextColor(getColor(R.color.learned));
+        } else super.onBackPressed();
+    }
+
+    @Override
+    protected void onDestroy() {
+        NotificationService.enqueueWork(context);
+        isPaukerActive = false;
+        super.onDestroy();
+    }
+
+    @Override
+    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions,
                                            @NonNull int[] grantResults) {
-        if (requestCode == RQ_WRITE_EXT) {
-            if ((grantResults.length > 0) && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
-                openLesson();
-            }
+        if (requestCode == RQ_WRITE_EXT_OPEN
+                && (grantResults.length > 0) && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
+            openLesson();
+        }
+        if (requestCode == RQ_WRITE_EXT_SAVE
+                && (grantResults.length > 0) && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
+            saveLesson(REQUEST_CODE_SAVE_DIALOG_NORMAL);
         }
     }
 
     @Override
     protected void onActivityResult(int requestCode, int resultCode, Intent data) {
-        if (requestCode == Constants.REQUEST_CODE_SAVE_DIALOG_NORMAL) {
+        if (requestCode == REQUEST_CODE_SAVE_DIALOG_NORMAL) {
             if (resultCode == RESULT_OK) {
                 showToast((Activity) context, R.string.saving_success, Toast.LENGTH_SHORT);
                 paukerManager.setSaveRequired(false);
                 modelManager.showExpireToast(context);
             }
             invalidateOptionsMenu();
-        } /*else if (requestCode == Constants.REQUEST_CODE_SYNC_DIALOG) {
-            if (resultCode == RESULT_OK) {
-                Log.d("SyncLesson", "Synchro erfolgreich");
-            } else {
-                Log.d("SyncLesson", "Synchro nicht erfolgreich");
-                PaukerManager.showToast((Activity) context, R.string.error_synchronizing, Toast.LENGTH_SHORT);
-            }
-        } else if (resultCode == RESULT_OK && requestCode == Constants.REQUEST_CODE_SYNC_DIALOG_BEFORE_OPEN) {
-            startActivity(new Intent(context, LessonImportActivity.class));
-        }*/ else if (requestCode == Constants.REQUEST_CODE_SAVE_DIALOG_NEW_LESSON && resultCode == RESULT_OK) {
+        } else if (requestCode == Constants.REQUEST_CODE_SAVE_DIALOG_NEW_LESSON && resultCode == RESULT_OK) {
             createNewLesson();
         } else if (requestCode == Constants.REQUEST_CODE_SAVE_DIALOG_OPEN && resultCode == RESULT_OK) {
             startActivity(new Intent(context, LessonImportActivity.class));
@@ -351,68 +386,22 @@ public class MainMenu extends AppCompatActivity {
      * @param requestCode Wird für onActivityResult benötigt
      */
     private void saveLesson(final int requestCode) {
-        startActivityForResult(new Intent(context, SaveDialog.class), requestCode);
+        if (checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
+            checkPermission(RQ_WRITE_EXT_SAVE);
+        } else {
+            startActivityForResult(new Intent(context, SaveDialog.class), requestCode);
+        }
     }
 
     /**
      * Fragt, wenn notwendig, die Permission ab und zeigt davor einen passenden Infodialog an.
      */
     private void openLesson() {
-        final SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(context);
-        AlertDialog.Builder builder = new AlertDialog.Builder(context);
-        builder.setTitle(R.string.app_name)
-                .setNegativeButton(R.string.not_now, new DialogInterface.OnClickListener() {
-                    @Override
-                    public void onClick(DialogInterface dialog, int which) {
-                        dialog.dismiss();
-                    }
-                })
-                .setPositiveButton(R.string.next, new DialogInterface.OnClickListener() {
-                    @Override
-                    public void onClick(DialogInterface dialog, int which) {
-                        pref.edit().putBoolean("FirstTime", false).apply();
-                        requestPermission();
-                        dialog.dismiss();
-                    }
-                });
-        if (checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE)
-                != PackageManager.PERMISSION_GRANTED) {
-            if (shouldShowRequestPermissionRationale(Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
-                builder.setMessage(R.string.write_permission_rational_message);
-            } else {
-                if (pref.getBoolean("FirstTime", true)) {
-                    builder.setMessage(R.string.write_permission_info_message);
-                } else {
-                    builder.setMessage(R.string.write_permission_rational_message)
-                            .setPositiveButton(R.string.settings, new DialogInterface.OnClickListener() {
-                                @Override
-                                public void onClick(DialogInterface dialog, int which) {
-                                    Intent intent = new Intent();
-                                    intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
-                                    Uri uri = Uri.fromParts("package", getPackageName(), null);
-                                    intent.setData(uri);
-                                    startActivity(intent);
-                                    dialog.dismiss();
-                                }
-                            });
-                }
-            }
-            AlertDialog dialog = builder.create();
-            dialog.show();
+        if (checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
+            checkPermission(RQ_WRITE_EXT_OPEN);
         } else {
-            /*if (settingsManager.getBoolPreference(context, AUTO_SYNC)) {
-                String accessToken = PreferenceManager.getDefaultSharedPreferences(context)
-                        .getString(Constants.DROPBOX_ACCESS_TOKEN, null);
-                if (accessToken != null) {
-                    File[] files = paukerManager.listFiles(context);
-                    Intent syncIntent = new Intent(context, SyncDialog.class);
-                    syncIntent.putExtra(SyncDialog.ACCESS_TOKEN, accessToken);
-                    syncIntent.putExtra(SyncDialog.FILES, files);
-                    startActivityForResult(syncIntent, Constants.REQUEST_CODE_SYNC_DIALOG_BEFORE_OPEN);
-                }
-            } else {*/
-            if (paukerManager.isSaveRequired()) {
-                builder = new AlertDialog.Builder(context);
+           if (paukerManager.isSaveRequired()) {
+                AlertDialog.Builder builder = new AlertDialog.Builder(context);
                 builder.setTitle(R.string.lesson_not_saved_dialog_title)
                         .setMessage(R.string.save_lesson_before_question)
                         .setPositiveButton(R.string.save, new DialogInterface.OnClickListener() {
@@ -431,7 +420,6 @@ public class MainMenu extends AppCompatActivity {
                 builder.create().show();
             } else
                 startActivity(new Intent(context, LessonImportActivity.class));
-            //}
         }
     }
 
@@ -447,8 +435,55 @@ public class MainMenu extends AppCompatActivity {
         showToast((Activity) context, R.string.new_lession_created, Toast.LENGTH_SHORT);
     }
 
+    /**
+     * Fragt den Nutzer nach der Erlaubnis.
+     * @param requestCode Welche Funktion danach ausgeführt werden soll.
+     */
+    private void checkPermission(final int requestCode) {
+        final SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(context);
+        AlertDialog.Builder builder = new AlertDialog.Builder(context);
+        builder.setTitle(R.string.app_name)
+                .setPositiveButton(R.string.next, new DialogInterface.OnClickListener() {
+                    @Override
+                    public void onClick(DialogInterface dialog, int which) {
+                        pref.edit().putBoolean("FirstTime", false).apply();
+                        requestPermissions(new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE},
+                                requestCode);
+                        dialog.dismiss();
+                    }
+                })
+                .setNeutralButton(R.string.not_now, new DialogInterface.OnClickListener() {
+                    @Override
+                    public void onClick(DialogInterface dialog, int which) {
+                        dialog.dismiss();
+                    }
+                });
+        if (shouldShowRequestPermissionRationale(Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
+            builder.setMessage(R.string.write_permission_rational_message);
+        } else {
+            if (pref.getBoolean("FirstTime", true)) {
+                builder.setMessage(R.string.write_permission_info_message);
+            } else {
+                builder.setMessage(R.string.write_permission_rational_message)
+                        .setPositiveButton(R.string.settings, new DialogInterface.OnClickListener() {
+                            @Override
+                            public void onClick(DialogInterface dialog, int which) {
+                                Intent intent = new Intent();
+                                intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
+                                Uri uri = Uri.fromParts("package", getPackageName(), null);
+                                intent.setData(uri);
+                                startActivity(intent);
+                                dialog.dismiss();
+                            }
+                        });
+            }
+        }
+        AlertDialog dialog = builder.create();
+        dialog.show();
+    }
+
     public void mSaveFileClicked(@Nullable MenuItem ignored) {
-        saveLesson(Constants.REQUEST_CODE_SAVE_DIALOG_NORMAL);
+        saveLesson(REQUEST_CODE_SAVE_DIALOG_NORMAL);
     }
 
     /**
@@ -475,7 +510,7 @@ public class MainMenu extends AppCompatActivity {
                         dialog.cancel();
                     }
                 })
-                .setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
+                .setNeutralButton(R.string.cancel, new DialogInterface.OnClickListener() {
                     @Override
                     public void onClick(DialogInterface dialog, int which) {
                         dialog.cancel();
@@ -495,7 +530,7 @@ public class MainMenu extends AppCompatActivity {
                             saveLesson(Constants.REQUEST_CODE_SAVE_DIALOG_NEW_LESSON);
                         }
                     })
-                    .setNegativeButton(R.string.no, new DialogInterface.OnClickListener() {
+                    .setNeutralButton(R.string.no, new DialogInterface.OnClickListener() {
                         @Override
                         public void onClick(DialogInterface dialog, int which) {
                             createNewLesson();
@@ -535,7 +570,7 @@ public class MainMenu extends AppCompatActivity {
                         dialog.cancel();
                     }
                 })
-                .setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
+                .setNeutralButton(R.string.cancel, new DialogInterface.OnClickListener() {
                     @Override
                     public void onClick(DialogInterface dialog, int which) {
                         dialog.cancel();
@@ -543,4 +578,51 @@ public class MainMenu extends AppCompatActivity {
                 });
         builder.create().show();
     }
+
+    /**
+     * Erstellt alle Channels
+     */
+    public void createNotificationChannels() {
+        // Für Notification
+        createNotificationChannel(getString(R.string.channel_notify_name_other),
+                null,
+                NotificationManager.IMPORTANCE_DEFAULT,
+                NOTIFICATION_CHANNEL_ID,
+                true);
+
+        // Timerbar
+        createNotificationChannel(getString(R.string.channel_timerbar_name),
+                getString(R.string.channel_timerbar_description),
+                NotificationManager.IMPORTANCE_LOW,
+                TIMER_BAR_CHANNEL_ID,
+                false);
+
+        // Falls der TimerChannel noch existiert
+        NotificationManager notificationManager = getSystemService(NotificationManager.class);
+        if (notificationManager != null) {
+            notificationManager.deleteNotificationChannel("Timers");
+            Log.d("MainMenu::createNotificationChannel", "Timer channel deleted");
+        }
+    }
+
+    /**
+     * Hilfsmethode zum Erstellen der Channels
+     */
+    private void createNotificationChannel(String channelName, String description, int importance, String ID, boolean playSound) {
+        NotificationChannel channel = new NotificationChannel(ID, channelName, importance);
+
+        if (description != null) {
+            channel.setDescription(description);
+        }
+
+        if (!playSound) {
+            channel.setSound(null, null);
+        }
+
+        NotificationManager notificationManager = getSystemService(NotificationManager.class);
+        if (notificationManager != null) {
+            notificationManager.createNotificationChannel(channel);
+        }
+        Log.d("AlamNotificationReceiver::createNotificationChannel", "Channel created: " + channelName);
+    }
 }
\ No newline at end of file
