@@ -14,6 +14,7 @@ import android.preference.PreferenceManager;
 import android.support.annotation.Nullable;
 import android.view.MotionEvent;
 import android.view.View;
+import android.widget.Button;
 import android.widget.ProgressBar;
 import android.widget.RelativeLayout;
 import android.widget.TextView;
@@ -52,6 +53,9 @@ import static com.daniel.mobilepauker2.PaukerManager.showToast;
 public class SyncDialog extends Activity {
     public static final String ACCESS_TOKEN = "ACCESS_TOKEN";
     public static final String FILES = "FILES";
+    public static final String SYNC_ALL_ACTION = "SYNC_ALL_ACTION";
+    public static final String UPLOAD_FILE_ACTION = "UPLOAD_FILE_ACTION";
+    public static final String SYNC_FILE_ACTION = "SYNC_FILE_ACTION";
     private final Context context = this;
     private final ModelManager modelManager = ModelManager.instance();
     private final PaukerManager paukerManager = PaukerManager.instance();
@@ -60,6 +64,8 @@ public class SyncDialog extends Activity {
     private TimerTask timerTask;
     private NetworkStateReceiver networkStateReceiver;
 
+    private Button cancelbutton;
+
     // Hintergrundtasks
     private List<AsyncTask> tasks = new ArrayList<>();
 
@@ -98,34 +104,108 @@ public class SyncDialog extends Activity {
 
         DropboxClientFactory.init(accessToken);
         Serializable serializableExtra = intent.getSerializableExtra(FILES);
-        if (serializableExtra instanceof File[]) {
-            RelativeLayout progressBar = findViewById(R.id.pFrame);
-            progressBar.setVisibility(View.VISIBLE);
-            TextView title = findViewById(R.id.pTitle);
-            title.setText(R.string.synchronizing);
-            files = (File[]) serializableExtra;
-            startTimer();
-            loadData();
+        startSync(intent, serializableExtra);
+    }
+
+    private void startSync(Intent intent, Serializable serializableExtra) {
+        String action = intent.getAction();
+        if (SYNC_ALL_ACTION.equals(action) && serializableExtra instanceof File[]) {
+            syncAllFiles((File[]) serializableExtra);
         } else if (serializableExtra instanceof File) {
-            Log.d("SyncDialog:onCreate", "Upload just one file");
-            List<File> list = new ArrayList<>();
-            File file = (File) serializableExtra;
-            if (file.exists()) {
-                Log.d("SyncDialog:onCreate", "file exists");
-                list.add((File) serializableExtra);
-                uploadFiles(list);
-                finishDialog(RESULT_OK);
-            } else {
-                Log.d("SyncDialog:onCreate", "file does not exist");
-                showToast((Activity) context, R.string.error_file_not_found, Toast.LENGTH_LONG);
-                finishDialog(RESULT_CANCELED);
-            }
+            syncFile((File) serializableExtra, action);
         } else {
             Log.d("SyncDialog::OnCreate", "Synchro mit falschem Extra gestartet");
             finishDialog(RESULT_CANCELED);
         }
     }
 
+    /**
+     * Synchronisiert alle Files mit denen auf Dropbox.
+     * @param serializableExtra Lokale Files die synchronisiert werden sollen
+     */
+    private void syncAllFiles(File[] serializableExtra) {
+        showProgressbar();
+        files = serializableExtra;
+        startTimer();
+        loadData();
+    }
+
+    /**
+     * Bei {@link SyncDialog#UPLOAD_FILE_ACTION UPLOAD_ACTION} wird das File hochgeladen.
+     * Dabei wird nicht überprüft ob das neuer oder älter ist, als das File auf Dropbox. <br>
+     * Bei {@link SyncDialog#SYNC_FILE_ACTION SYNC_FILE_ACTION} wird das File mit dem gleichnamigen
+     * auf Dropbox verglichen. Ist das auf Dropbox neuer, wird dieses heruntergeladen.
+     * @param serializableExtra File, das synchronisiert werden soll
+     * @param action Spezifiziert was mit dem File geschehen soll
+     */
+    private void syncFile(final File serializableExtra, String action) {
+        List<File> list = new ArrayList<>();
+        if (serializableExtra.exists()) {
+            Log.d("SyncDialog:syncFile", "File exists");
+            Log.d("SyncDialog:syncFile", "Syncaction: " + action);
+            if (UPLOAD_FILE_ACTION.equals(action)) {
+                Log.d("SyncDialog:syncFile", "Upload just one file");
+                list.add(serializableExtra);
+                uploadFiles(list);
+                finishDialog(RESULT_OK);
+            } else if (SYNC_FILE_ACTION.equals(action)) {
+                Log.d("SyncDialog:syncFile", "Download just one file");
+                showProgressbar();
+                showCancelButton();
+                AsyncTask<File, Void, Metadata> getFileMetadataTask =
+                        new GetFileMetadataTask(DropboxClientFactory.getClient(), new GetFileMetadataTask.Callback() {
+                    @Override
+                    public void onDataLoaded(Metadata metadata) {
+                        Log.d("SyncDialog:syncFile::onDataLoaded", "Data loaded");
+                        if (metadata instanceof FileMetadata) {
+                            FileMetadata fileMetadata = (FileMetadata) metadata;
+                            if (serializableExtra.lastModified() < fileMetadata.getClientModified().getTime()) {
+                                Log.d("SyncDialog:syncFile::onDataLoaded", "File wird runtergeladen");
+                                List<FileMetadata> metadataList = new ArrayList<>();
+                                metadataList.add(fileMetadata);
+                                downloadFiles(metadataList, (int) fileMetadata.getSize());
+                            } else {
+                                Log.d("SyncDialog:syncFile::onDataLoaded", "File wird NICHT runtergeladen");
+                                finishDialog(RESULT_CANCELED);
+                            }
+                        } else if (metadata instanceof DeletedMetadata) {
+                            showToast((Activity) context, "Datei ist nicht länger verfügbar auf Dropbox", Toast.LENGTH_LONG);
+                            finishDialog(RESULT_CANCELED);
+                        }
+                    }
+
+                    @Override
+                    public void onError(DbxException e) {
+                        showToast((Activity) context, R.string.error_synchronizing, Toast.LENGTH_LONG);
+                        finishDialog(RESULT_CANCELED);
+                    }
+                }).execute(serializableExtra);
+                tasks.add(getFileMetadataTask);
+            } else {
+                Log.d("SyncDialog:syncFile", "File does not exist");
+                showToast((Activity) context, R.string.error_synchronizing, Toast.LENGTH_LONG);
+                finishDialog(RESULT_CANCELED);
+            }
+        } else {
+            Log.d("SyncDialog:syncFile", "File does not exist");
+            showToast((Activity) context, R.string.error_file_not_found, Toast.LENGTH_LONG);
+            finishDialog(RESULT_CANCELED);
+        }
+    }
+
+    private void showProgressbar() {
+        RelativeLayout progressBar = findViewById(R.id.pFrame);
+        progressBar.setVisibility(View.VISIBLE);
+        TextView title = findViewById(R.id.pTitle);
+        title.setText(R.string.synchronizing);
+    }
+
+    private void showCancelButton() {
+        cancelbutton = findViewById(R.id.cancel_button);
+        cancelbutton.setVisibility(View.VISIBLE);
+        Log.d("SyncDialog::showCancelButton", "Button is enabled: " + cancelbutton.isEnabled());
+    }
+
     /**
      * Sucht nach allen Dateien im Dropboxordner, validiert sie und ruft dann
      * {@link SyncDialog#syncWithFiles(List, Map) synWithFiles()} auf.
@@ -175,12 +255,12 @@ public class SyncDialog extends Activity {
                         R.string.simple_error_message,
                         Toast.LENGTH_SHORT);
                 cancelTasks();
-                if (e.getRequestId().equals("401")) {
+                if (e.getRequestId() != null && e.getRequestId().equals("401")) {
                     AlertDialog.Builder builder = new AlertDialog.Builder(context);
                     builder.setTitle("Dropbox token is invalid!")
                             .setMessage("There is something wrong with the dropbox token. Maybe it is " +
                                     "solved by the next try. If this doesn't work" +
-                                    "please contact me.")
+                                    "please get in contact.")
                             .setPositiveButton(R.string.ok, null)
                             .setNeutralButton("Send E-Mail", new DialogInterface.OnClickListener() {
                                 @Override
@@ -322,20 +402,22 @@ public class SyncDialog extends Activity {
                 new DownloadFileTask.Callback() {
                     @Override
                     public void onDownloadStartet() {
+                        Log.d("SyncDialog:downloadFiles", "Download startet");
                         progressBar.setMax(downloadSize);
                         progressBar.setIndeterminate(false);
                     }
 
                     @Override
                     public void onProgressUpdate(FileMetadata metadata) {
+                        Log.d("SyncDialog:downloadFiles", "Download update: " + progressBar.getProgress() + metadata.getSize());
                         progressBar.setProgress((int) (progressBar.getProgress() + metadata.getSize()));
                     }
 
                     @Override
                     public void onDownloadComplete(File[] result) {
-                        setResult(RESULT_OK);
+                        Log.d("SyncDialog:downloadFiles", "Download complete");
                         progressBar.setIndeterminate(true);
-                        finish();
+                        finishDialog(RESULT_OK);
                     }
 
                     @Override
@@ -447,6 +529,16 @@ public class SyncDialog extends Activity {
 
     @Override
     public boolean dispatchTouchEvent(MotionEvent ev) {
+        Log.d("SyncDialog::TouchEvent", "Touched");
+        if(cancelbutton != null) {
+            int[] pos = new int[2];
+            cancelbutton.getLocationInWindow(pos);
+            if ((ev.getY() <= (pos[1] + cancelbutton.getHeight()) && ev.getX() > pos[0])
+                    && (ev.getY() > pos[1] && ev.getX() <= (pos[0] + cancelbutton.getWidth()))
+                    && ev.getAction() == MotionEvent.ACTION_UP) {
+                cancelClicked(cancelbutton);
+            }
+        }
         return false;
     }
 
@@ -468,6 +560,13 @@ public class SyncDialog extends Activity {
         timeout.schedule(timerTask, 60000);
     }
 
+    public void cancelClicked(View view) {
+        Log.d("SyncDialog::cancelClicked", "Cancel Sync");
+        view.setEnabled(false);
+        PaukerManager.showToast((Activity) context, R.string.synchro_canceled_by_user, Toast.LENGTH_LONG);
+        finishDialog(RESULT_CANCELED);
+    }
+
     private void cancelTasks() {
         for (AsyncTask task : tasks) {
             if (task != null && task.getStatus() != AsyncTask.Status.FINISHED) {
