@@ -33,6 +33,7 @@ import android.widget.Toast;
 
 import com.daniel.mobilepauker2.PaukerManager;
 import com.daniel.mobilepauker2.R;
+import com.daniel.mobilepauker2.model.pauker_native.Batch;
 import com.daniel.mobilepauker2.model.pauker_native.Card;
 import com.daniel.mobilepauker2.model.pauker_native.Font;
 import com.daniel.mobilepauker2.model.pauker_native.Lesson;
@@ -79,9 +80,8 @@ import static com.daniel.mobilepauker2.model.SettingsManager.Keys.RETURN_FORGOTT
  */
 
 public class ModelManager {
-    private static ModelManager instance;
     private static final PaukerManager paukerManager = PaukerManager.instance();
-
+    private static ModelManager instance;
     private final List<FlashCard> mCurrentPack = new ArrayList<>();
     private final SettingsManager settingsManager = SettingsManager.instance();
     private Lesson mLesson = null;
@@ -99,51 +99,6 @@ public class ModelManager {
         return instance;
     }
 
-    /**
-     * the learning phases
-     */
-    public enum LearningPhase {
-
-        /**
-         * not learning
-         */
-        NOTHING,
-        /**
-         * Just browsing the new cards - not learning
-         */
-        BROWSE_NEW,
-
-        /**
-         * Not using USTM or STM memory and timers
-         */
-        SIMPLE_LEARNING,
-
-        /**
-         * the phase of filling the ultra short term memory
-         */
-        FILLING_USTM,
-        /**
-         * the phase of waiting for the ultra short term memory
-         */
-        WAITING_FOR_USTM,
-        /**
-         * the phase of repeating the ultra short term memory
-         */
-        REPEATING_USTM,
-        /**
-         * the phase of waiting for the short term memory
-         */
-        WAITING_FOR_STM,
-        /**
-         * the phase of repeating the short term memory
-         */
-        REPEATING_STM,
-        /**
-         * the phase of repeating the long term memory
-         */
-        REPEATING_LTM
-    }
-
     private void setupCurrentPack(Context context) {
 
         Iterator<Card> cardIterator = null;
@@ -189,9 +144,8 @@ public class ModelManager {
                 }
 
                 case WAITING_FOR_STM: {
-                    //noinspection UnusedAssignment
                     cardIterator = mLesson.getShortTermList().listIterator(); // Need to put something in the iterator for requery
-                    return;
+                    break;
                 }
 
                 case REPEATING_STM: {
@@ -221,7 +175,7 @@ public class ModelManager {
             mCurrentPack.clear();
             switch (stackIndex) {
                 case 0:
-                    cardIterator = mLesson.getCards().iterator();
+                    cardIterator = mLesson.getSummaryBatch().getCards().iterator();
                     break;
                 case 1:
                     cardIterator = mLesson.getUnlearnedBatch().getCards().iterator();
@@ -253,7 +207,7 @@ public class ModelManager {
     public void addCard(FlashCard flashCard, String sideA, String sideB) {
         flashCard.setSideAText(sideA);
         flashCard.setSideBText(sideB);
-        mLesson.getUnlearnedBatch().addCard(flashCard);
+        getLesson().addCard(flashCard);
     }
 
     public List<BatchStatistics> getBatchStatistics() {
@@ -505,7 +459,7 @@ public class ModelManager {
                 String date = DateFormat.format("dd.MM.yyyy HH:mm", cal).toString();
                 String text = context.getString(R.string.next_expire_date);
                 text = text.concat(" ").concat(date);
-                PaukerManager.showToast((Activity)context, text, Toast.LENGTH_LONG * 2);
+                PaukerManager.showToast((Activity) context, text, Toast.LENGTH_LONG * 2);
             }
         } catch (MalformedURLException ignored) {
         }
@@ -708,26 +662,66 @@ public class ModelManager {
         setLesson(newLesson);
     }
 
+    /**
+     * Sortiert den Batch, aber aktuellisiert nicht den aktuellen.
+     * @param stackIndex    Index des Batchs
+     * @param sortByElement Nach diesem Element wird soriert
+     * @param asc_direction In welche Richtung sortiert werden soll
+     */
+    public void sortBatch(int stackIndex, Card.Element sortByElement, boolean asc_direction) {
+        Batch batch;
+        switch (stackIndex) {
+            case 0:
+                batch = mLesson.getSummaryBatch();
+                break;
+            case 1:
+                batch = mLesson.getUnlearnedBatch();
+                break;
+            default:
+                batch = mLesson.getLongTermBatch(stackIndex - 2);
+        }
+        if (batch != null)
+            batch.sortCards(sortByElement, asc_direction);
+    }
+
+    /**
+     * Löscht die Karte an der übergebenen Position und entfernt diese aus dem aktuellen Stack.
+     * @param position Position der Karte
+     * @return True, wenn Karte erfolgreich gelöscht wurde
+     */
     public boolean deleteCard(int position) {
 
-        mCurrentCard = mCurrentPack.get(position);
+        Card card = mCurrentPack.get(position);
 
+        if (deleteCard(card)) return false;
+
+        mCurrentPack.remove(position);
+
+        return true;
+    }
+
+    /**
+     * Löscht die Karte, aber entfernt diese nicht aus dem aktuellen Stack.
+     * @param card Zu löschende Karte
+     * @return True, wenn Karte erfolgreich gelöscht wurde
+     */
+    public boolean deleteCard(Card card) {
         Log.d("AndyPaukerApplication::deleteCard", "entry");
 
-        if (mCurrentCard.isLearned()) {
-            int batchNumber = mCurrentCard.getLongTermBatchNumber();
+        if (card.isLearned()) {
+            int batchNumber = card.getLongTermBatchNumber();
             LongTermBatch longTermBatch = mLesson.getLongTermBatch(batchNumber);
-            if (longTermBatch.removeCard(mCurrentCard)) {
+            if (longTermBatch.removeCard(card)) {
                 Log.d("AndyPaukerApplication::deleteCard", "Deleted from long term batch" + batchNumber);
             } else {
                 Log.e("AndyPaukerApplication::deleteCard", "Card not in long term batch" + batchNumber);
             }
         } else {
-            if (mLesson.getUnlearnedBatch().removeCard(mCurrentCard)) {
+            if (mLesson.getUnlearnedBatch().removeCard(card)) {
                 Log.d("AndyPaukerApplication::deleteCard", "Deleted from unlearned batch");
-            } else if (mLesson.getUltraShortTermList().remove(mCurrentCard)) {
+            } else if (mLesson.getUltraShortTermList().remove(card)) {
                 Log.d("AndyPaukerApplication::deleteCard", "Deleted from ultra short term batch");
-            } else if (mLesson.getShortTermList().remove(mCurrentCard)) {
+            } else if (mLesson.getShortTermList().remove(card)) {
                 Log.d("AndyPaukerApplication::deleteCard", "Deleted from short term batch");
             } else {
                 Log.e("AndyPaukerApplication::deleteCard", "Could not delete card from unlearned batch  ");
@@ -735,8 +729,12 @@ public class ModelManager {
             }
         }
 
-        mCurrentPack.remove(position);
-
+        if (mLesson.getSummaryBatch().removeCard(card)) {
+            Log.d("AndyPaukerApplication::deleteCard", "Deleted from summary batch");
+        } else {
+            Log.e("AndyPaukerApplication::deleteCard", "Could not delete card from summary batch  ");
+            return false;
+        }
         return true;
     }
 
@@ -745,14 +743,6 @@ public class ModelManager {
         setupCurrentPack(context);
     }
 
-    public void setLesson(Lesson lesson) {
-        mLesson = lesson;
-    }
-
-    public void setDescription(String s) {
-        mLesson.setDescription(s);
-    }
-
     public LearningPhase getLearningPhase() {
         return mLearningPhase;
     }
@@ -761,6 +751,10 @@ public class ModelManager {
         return mLesson.getDescription();
     }
 
+    public void setDescription(String s) {
+        mLesson.setDescription(s);
+    }
+
     public FlashCard getCard(int position) {
         if (position < 0 || position >= mCurrentPack.size()) {
             return null;
@@ -829,8 +823,6 @@ public class ModelManager {
         return mLesson.getCards().size();
     }
 
-    //TODO Have an enum BATCH_TYPE and pass that into a single getBatchSize(BatchType)
-
     public int getExpiredCardsSize() {
         return mLesson.getNumberOfExpiredCards();
     }
@@ -839,6 +831,8 @@ public class ModelManager {
         return mLesson.getUnlearnedBatch().getCards().size();
     }
 
+    //TODO Have an enum BATCH_TYPE and pass that into a single getBatchSize(BatchType)
+
     public int getUltraShortTermMemorySize() {
         return mLesson.getUltraShortTermList().size();
     }
@@ -855,6 +849,10 @@ public class ModelManager {
         return mLesson;
     }
 
+    public void setLesson(Lesson lesson) {
+        mLesson = lesson;
+    }
+
     /*
      * Shuffle the card pack
      */
@@ -880,4 +878,78 @@ public class ModelManager {
         return mLearningPhase == LearningPhase.REPEATING_LTM
                 && settingsManager.getBoolPreference(context, LEARN_NEW_CARDS_RANDOMLY);
     }
+
+    public void forgetCard(Card card) {
+        if (!card.isLearned())
+            return;
+
+        Batch batch = getBatchOfCard(card);
+        int index = batch.indexOf(card);
+        if (index != -1) {
+            mLesson.forgetCards(batch, new int[]{index});
+        }
+    }
+
+    public void instantRepeatCard(Card card) {
+        Batch batch = getBatchOfCard(card);
+        int index = batch.indexOf(card);
+        if (index != -1) {
+            mLesson.instantRepeatCards(batch, new int[]{index});
+        }
+    }
+
+    private Batch getBatchOfCard(Card card) {
+        int batchNumber = card.getLongTermBatchNumber();
+        if (batchNumber == 0)
+            return mLesson.getSummaryBatch();
+        else if (batchNumber == 1)
+            return mLesson.getUnlearnedBatch();
+        else
+            return mLesson.getLongTermBatch(batchNumber);
+    }
+
+    /**
+     * the learning phases
+     */
+    public enum LearningPhase {
+
+        /**
+         * not learning
+         */
+        NOTHING,
+        /**
+         * Just browsing the new cards - not learning
+         */
+        BROWSE_NEW,
+
+        /**
+         * Not using USTM or STM memory and timers
+         */
+        SIMPLE_LEARNING,
+
+        /**
+         * the phase of filling the ultra short term memory
+         */
+        FILLING_USTM,
+        /**
+         * the phase of waiting for the ultra short term memory
+         */
+        WAITING_FOR_USTM,
+        /**
+         * the phase of repeating the ultra short term memory
+         */
+        REPEATING_USTM,
+        /**
+         * the phase of waiting for the short term memory
+         */
+        WAITING_FOR_STM,
+        /**
+         * the phase of repeating the short term memory
+         */
+        REPEATING_STM,
+        /**
+         * the phase of repeating the long term memory
+         */
+        REPEATING_LTM
+    }
 }
